/*
 * Trffner mittels Morsecode an der Klingel bettigen
 * (und bei dabei nicht klingeln)
 *
 * Created: 08.01.2014 19:45:12
 * Author: Alex Handfest
 * www.alex-handfest.de
 * 
 * ATtiny85 -20SU
 * 
 * PB0 OUT Klingel schalten (zur Klingelunterdrckung bei richtigem Code)
 * PB2  IN Lernmodus (zum Aktivieren mit Jumperkabel auf GND) <- nicht aktiv!!
 * PB3  IN Klingel abfragen 
 * PB4 OUT Trffner schalten
 * 
 * mit F_CPU 500000 sehr str anfllig und Kondensator hlt nur 1-2 Sekunden
 * mit F_CPU  16000 luft sauber und Kondensator hlt 5-6 Sekunden
 * Brown out Detektion ist mit 2,7V an! Sonst macht der Controller schmarn
 */ 


//#define F_CPU		500000		// internal OSC   4 MHz / 8 = 500 kHz
//#define F_CPU		128000		// Watchdog OSC 128 kHz / 1 = 128 kHz	- ISP Takt runter setzen !!
#define F_CPU		 16000		// Watchdog OSC 128 kHz / 8 =  16 kHz	- ISP Takt runter setzen !!


#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/eeprom.h>
//#include <util/delay.h>


#define KLINGEL_AN		do{PORTB |=  (1<<PB0); TCCR1 = 0;}while(0)		// und Timer aus
#define KLINGEL_AUS		(PORTB &= ~(1<<PB0))
#define TUERE_AN		(PORTB |=  (1<<PB4))
#define TUERE_AUS		(PORTB &= ~(1<<PB4))



// ************************************************************************
//		
//							EINSTELLUNGEN
//		
//		hier den Aktivierungs Code einstellen !!!
//		0 = kurzer Puls
//		1 = langer Puls
//		die erste Stelle muss kurz sein, wegen Klingelunterdrckung!
//		c_laenge = Lnge des Codes
//		
unsigned char code[20]	= {0, 1, 0, 1, 0, 1, 0, 1, 0, 1};
unsigned char c_laenge	= 6;
// 
// ************************************************************************


unsigned char k_status	= 0;		// 1 = jemand klingelt
unsigned char k_count	= 0;		// Klingel Zhler
unsigned char p_count 	= 0;		// Pausen Zhler
unsigned char c_point	= 0;		// Pointer auf Code Array
unsigned char c_ok		= 0;		// berprfung der Eingabe
	
//char eeFooArray[10] EEMEM = {0,0,1,0,0,0,0,0,0,0};



int main(void){
	
	// Ausgnge & Pull-up
	DDRB   = (1<<PB0) | (1<<PB4);
	PORTB |= (1<<PB2) | (1<<PB3);
	
	
	// Initialisierung Timer1
	//GTCCR |= (0<<PSR1) | (0<<PWM1B) | (0<<COM1B1) | (0<<COM1B0) | (0<<FOC1B) | (0<<FOC1A);
	//TCCR1 |= (0<<CTC1) | (0<<PWM1A) | (0<<COM1A1) | (0<<COM1A0);
	TCCR1 |= (0<<CS13) | (0<<CS12) | (1<<CS11)  | (0<<CS10);		// Prescaler 2
	TIMSK |= (0<<OCIE1A) | (0<<OCIE1B) | (1<<TOIE1);				// Overflow Interrupt Enable
	
	// 500000 /   64 / 256 = 30,5
	// 500000 /  128 / 256 = 15,2
	//  16000 /    4 / 256 = 15,6
	//  16000 /    2 / 256 = 31,2

	
	// Interrupts aktivieren
	sei();
	
	
    while(1){
		
		// Klingeln abfragen
		// Klingelfrequenz 50Hz, nur positive Welle wird dedektiert -> in der while(1)
		
		if (!(PINB & (1<<PB3))) k_status = 1;
	}
	
}



// ************************************************************************ OVF Interrupt
// 16000 / 2 / 256 = 31,2
ISR(TIMER1_OVF_vect){
	
	
	if (k_status == 1){							// jemand hat geklingelt
		
		k_count++;
		p_count = 0;
		
		if (code[c_point] == 0){
			
			if (k_count > 3) KLINGEL_AN;		// kurze Pulse sofort abbrechen, wenn falsch
		}
	}
	else{
		
		if (k_count > 0){
			
			//code[c_point] = eeprom_read_byte(&eeFooArray[c_point]);
			
			if (k_count > 3){					// langer Klingel Puls
				
				if (code[c_point] == 1) c_ok++;
				else KLINGEL_AN;
			}
			else{								// kurzer Klingel Puls
			
				if (code[c_point] == 0) c_ok++;
				else KLINGEL_AN;
			}
			
			if (c_point == (c_laenge -1)){		// Code fertig
				
				if (c_ok == c_laenge) TUERE_AN;	// Tr auf
				else KLINGEL_AN;
			}
			
			c_point++;
			k_count = 0;
		}
		
		p_count++;
		if (p_count == 40) KLINGEL_AN;			// Pause zu lang
		
	}
	
	k_status = 0;
	
}